/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* real time tracking routines
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"
#include "iptrk.p"

typedef struct
{
  U_32   addr;
  volp_t volp;
} sort_addr_t;

PRIVATE treap_t *sort_bt;
PRIVATE int     entry_size;

/**************************************************************************
*
**************************************************************************/
PRIVATE void update_sort_entry(
  U_32 addr, U_64 vol_tot, U_64 vol_oth, U_64 vol_tcp, U_64 vol_udp)
{
  sort_addr_t entry, *p;

  int find_cmp_func(void *n)
  {
    return cmp_32(entry.addr, ((sort_addr_t *)n)->addr);
  }

  int append_cmp_func(void *n1, void *n2)
  {
    return cmp_32(((sort_addr_t *)n1)->addr, ((sort_addr_t *)n2)->addr);
  }

  entry.addr = addr;
  p = treap_find(&sort_bt, find_cmp_func);
  if(p == NULL)
  {
    entry.volp.tot = 0;
    entry.volp.oth = 0;
    entry.volp.tcp = 0;
    entry.volp.udp = 0;
    treap_append(&sort_bt, &entry, sizeof(sort_addr_t), append_cmp_func);

    p = treap_find(&sort_bt, find_cmp_func);
    if(p == NULL)
    {
      log_m('E', __func__, "::treap_find(): failed");
      exit(-1);
    }

    entry_size++;
  }

  p->volp.tot += vol_tot;
  p->volp.oth += vol_oth;
  p->volp.tcp += vol_tcp;
  p->volp.udp += vol_udp;
}

/**************************************************************************
*
**************************************************************************/
PRIVATE U_64 tally_port_vol(treap_t *t)
{
  port_t *n;
  U_64   v;

  if(t == NULL)
    return 0;

  n = TREAP_ENTRY(t);
  v = n->vol;
  v += tally_port_vol(t->l);
  v += tally_port_vol(t->r);

  return v;
}

/**************************************************************************
*
**************************************************************************/
PRIVATE int sort_cmp_func(const void *a, const void *b)
{
  return cmp_64(((sort_addr_t *)b)->volp.tot, ((sort_addr_t *)a)->volp.tot);
}

PUBLIC void display_sorted_grams(
  U_32 src, U_32 dst, U_16 vol, U_8 prot, U_16 port_s, U_16 port_d, int dir_f)
{
  char           str[STR_LEN + 1];
  ll_t           *ll;
  our_net_t      *our_net;
  sort_addr_t    *sort_tab = NULL;
  struct in_addr in;
  int            i = 0;

  void w_tree(treap_t *t)
  {
    if(t != NULL)
    {
      iptrk_t *n;
      U_64    a, b, c;

      w_tree(t->l);

      n = TREAP_ENTRY(t);
      a = tally_port_vol(n->oth_port_ll);
      b = tally_port_vol(n->tcp_port_ll);
      c = tally_port_vol(n->udp_port_ll);
      if(!dir_f) /*packets in*/
      {
        if(our_net->addr.addr == (n->addr.addr.d & our_net->addr.mask))
          update_sort_entry(n->addr.addr.d, n->vol, a, b, c);
      }
      else
      {
        if(our_net->addr.addr == (n->addr.addr.s & our_net->addr.mask))
          update_sort_entry(n->addr.addr.s, n->vol, a, b, c);
      }

      w_tree(t->r);
    }
  }

  void w_sort(treap_t *t)
  {
    if(t != NULL)
    {
      sort_addr_t *n;

      w_sort(t->l);

      n = TREAP_ENTRY(t);
      memcpy(&(sort_tab[i]), n, sizeof(sort_addr_t));
      i++;

      w_sort(t->r);
    }
  }

 /*
  * start
  */
  entry_size = 0;
  ll = boot.our_net_ll;
  for(;;)
  {
    our_net = ll_read_entry(&ll);
    if(our_net == NULL)
      break;

    w_tree(watch_bt);
  }

  sort_tab = alloc_mem(sizeof(sort_addr_t) * entry_size);
  if(sort_tab != NULL)
  {
    int i, s;

    w_sort(sort_bt); /*convert tree to table*/
    qsort(sort_tab, entry_size, sizeof(sort_addr_t), sort_cmp_func);

    wattrset(win_gram, COLOR_PAIR(COLOUR_WHITE) | A_BOLD);
    werase(win_gram);
    s = entry_size;
    if(s >= win_ysize_gram_len)
      s = win_ysize_gram_len;
    for(i = 0; i < s; i++)
    {
      in.s_addr = htonl(sort_tab[i].addr);
      sprintf(str, "%-15s %9llu %9llu %9llu %8llu",
        inet_ntoa(in),
        sort_tab[i].volp.tot,
        sort_tab[i].volp.tcp,
        sort_tab[i].volp.udp,
        sort_tab[i].volp.oth);
      wmove(win_gram, i, 0);
      waddstr(win_gram, str);
    }
    free(sort_tab);
  }
  else
    werase(win_gram);

  wrefresh(win_gram);
  treap_free(&sort_bt);
}
